unit UnitMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, StrUtils, DateUtils, Menus;

type
  TMainForm = class(TForm)
    Memo1: TMemo;
    Panel1: TPanel;
    OpenDialog: TOpenDialog;
    CloseTimer: TTimer;
    Panel2: TPanel;
    CloseButton: TButton;
    StartButton: TButton;
    MainMenu1: TMainMenu;
    N1: TMenuItem;
    ChooseVoice_1: TMenuItem;
    ChooseVoice_2: TMenuItem;
    ChooseVoice_3: TMenuItem;
    procedure StartButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CloseButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CloseTimerTimer(Sender: TObject);
  private
    { Private declarations }
    procedure StartConversionRegisters();
    function ConvertTimeToInterval(pList: TStrings; pLine: Integer): String;
    function GetTimeDifference(pTimeNow: String; pTimeThen: String): String;
    procedure AddLine(pLineOfText: String);
    procedure Print(pLineOfText: String);
    procedure Cls();
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  gCloseButtonPressed: Boolean;
  gCurrentDirectory: String;
  gRawMusicDirectory: String;
  gConvertedMusicDirectory: String;
  gTuneFileNameOnly: String;
  gMusicInList: TStringList;	        // Input stringlist for music file
  gMusicOutList: TstringList;           // Output stringlist for converted music file
  gLastTimeValue: Integer;              // Remember the last CentiSecond value that was read
  gTime: Integer;                       // Global Time Count
  gInterval: Integer;
  gLineCount: Integer;
implementation

{$R *.dfm}
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// Form Create
procedure TMainForm.FormCreate(Sender: TObject);
begin
	gMusicInList := TStringList.Create;
    gMusicOutList := TStringList.Create;
end;
//----------------------------------------------------------------------------
// Form Activate
procedure TMainForm.FormActivate(Sender: TObject);
begin
	OnActivate := Nil;
    gCloseButtonPressed := False;
    gCurrentDirectory := ExtractFileDir(Application.ExeName);
    gRawMusicDirectory := gCurrentDirectory + '\RAW Tunes\';
    gCOnvertedMusicDirectory := gCurrentDirectory + '\Converted Tunes\';
end;
//----------------------------------------------------------------------------
// Form Close Event
procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	gMusicInList.Free;
    gMusicOutList.Free;
end;
//----------------------------------------------------------------------------
// Start Button Event
procedure TMainForm.StartButtonClick(Sender: TObject);
begin
    StartButton.Enabled := False;                                       // Disable the start button
    gLastTimeValue := 0;
    gTime := 0;
    gInterval := 0;
    gLineCount := 0;
    gMusicOutList.Clear();

    OpenDialog.InitialDir := gRawMusicDirectory;
	OpenDialog.Execute();
    If (OpenDialog.FileName <> '') Then
    	Begin
    		gMusicInList.LoadFromFile(OpenDialog.FileName);             // Load the RAW music text file into a stringlist
            gTuneFileNameOnly := ExtractFileName(OpenDialog.FileName);  // Extract only the file name
            gTuneFileNameOnly := ChangeFileExt(gTuneFileNameOnly,'');   // Remove the file extension from teh filename

           MainForm.Caption := 'SID Music Reader: ' +  gTuneFileNameOnly;
            StartConversionRegisters();                                 // Convert it into register information. Creates a smaller data table
        End;
    If(gCloseButtonPressed = True) Then Close();
    StartButton.Enabled := True;                                       // Enable the start button
end;
//----------------------------------------------------------------------------
// Close Button Event
procedure TMainForm.CloseButtonClick(Sender: TObject);
begin
    gCloseButtonPressed := True;
    CloseTimer.Enabled := True;
	//Close();
end;
//----------------------------------------------------------------------------
// Convert the music file as Register changes.
// This creates a smaller data table to fit within the PICmicro
procedure TMainForm.StartConversionRegisters();
var
    LineIndex: Integer;
    LineOfText: String;
    Parse: TStringList;
    TickList: TStringList;

    iVoice1_Freq, iVoice2_Freq, iVoice3_Freq: Integer;

    sVoice1_Freq, sVoice1_PulseWidth, sVoice1_WaveForm, sVoice1_ADSR, sVoice1_Volume: String;
    sVoice2_Freq, sVoice2_PulseWidth, sVoice2_WaveForm, sVoice2_ADSR, sVoice2_Volume: String;
    sVoice3_Freq, sVoice3_PulseWidth, sVoice3_WaveForm, sVoice3_ADSR, sVoice3_Volume: String;

    sVoice1_FreqLo, sVoice1_PulseWidthLo, sVoice1_FreqHi, sVoice1_PulseWidthHi: String;
    sVoice2_FreqLo, sVoice2_PulseWidthLo, sVoice2_FreqHi, sVoice2_PulseWidthHi: String;
    sVoice3_FreqLo, sVoice3_PulseWidthLo, sVoice3_FreqHi, sVoice3_PulseWidthHi: String;

    sVoice1_AttackDecay, sVoice1_SustainRelease: String;
    sVoice2_AttackDecay, sVoice2_SustainRelease: String;
    sVoice3_AttackDecay, sVoice3_SustainRelease: String;

    sShadowVoice1_Freq, sShadowVoice1_PulseWidth, sShadowVoice1_WaveForm, sShadowVoice1_ADSR, sShadowVoice1_Volume: String;
    sShadowVoice2_Freq, sShadowVoice2_PulseWidth, sShadowVoice2_WaveForm, sShadowVoice2_ADSR, sShadowVoice2_Volume: String;
    sShadowVoice3_Freq, sShadowVoice3_PulseWidth, sShadowVoice3_WaveForm, sShadowVoice3_ADSR, sShadowVoice3_Volume: String;

    sShadowVoice1_FreqLo, sShadowVoice1_PulseWidthLo, sShadowVoice1_FreqHi, sShadowVoice1_PulseWidthHi: String;
    sShadowVoice2_FreqLo, sShadowVoice2_PulseWidthLo, sShadowVoice2_FreqHi, sShadowVoice2_PulseWidthHi: String;
    sShadowVoice3_FreqLo, sShadowVoice3_PulseWidthLo, sShadowVoice3_FreqHi, sShadowVoice3_PulseWidthHi: String;

    sShadowVoice1_AttackDecay, sShadowVoice1_SustainRelease: String;
    sShadowVoice2_AttackDecay, sShadowVoice2_SustainRelease: String;
    sShadowVoice3_AttackDecay, sShadowVoice3_SustainRelease: String;

    sShadowVoice1MusicLineString: String;       // Holds the complete line of values without comma's etc for Voice 1. For comparison with the previous line
    sShadowVoice2MusicLineString: String;       // Holds the complete line of values without comma's etc for Voice 1. For comparison with the previous line
    sShadowVoice3MusicLineString: String;       // Holds the complete line of values without comma's etc for Voice 1. For comparison with the previous line

    sVoice1MusicLine, sVoice2MusicLine, sVoice3MusicLine: String;
    iTemp: Integer;
    //iVoice1_Time, iVoice2_Time, iVoice3_Time: Integer;
    sVoice1, sVoice2, sVoice3: String;
    //iPreviousTime: Integer;
    tFirstLine: Boolean;
    iByteCount: Integer;
    tVoice1_Used, tVoice2_Used, tVoice3_Used: Boolean;
    sTickName: String;
    sRepTickName: String;

    sDelayInterval: String;
    sShadowDelayInterval: String;
    sPreviousDelayInterval: String;
    sTime: String;
    sCurrentTime: String;
    sPreviousTime: String;
    iInsertionCount: Integer;

    tCurrentTimeLoaded: Boolean;
    tPlaceATick: Boolean;
    sTickTemp: String;
    iTickIndex: Integer;
Const
    cFrequencyMultiplier = 2;                                     // Multiplies this by the note to make it higer pitched
begin

    Cls();          // Clear the display first
    Print('Converting ' + gTuneFileNameOnly + ' Please wait...');
//
// Move to the beginning of the music data
//
    LineIndex := 0;
    Repeat
        LineOfText := gMusicInList[LineIndex];
        //Print(LineOfText);
        If(AnsiPos('===',LineOfText) > 0) Then                      // Locate the line containing a list of '=' characters
            begin
                Inc(LineIndex);                                     // Skip over the line
                //Inc(LineIndex);                                     // Skip over the first line of music because it has a duff ADSR apparently
                Break;                                              // Exit the loop
            end;
        Inc(LineIndex);
    Until(LineIndex >= gMusicInList.Count) OR (gCloseButtonPressed = True);
    Parse := TStringList.Create;
    TickList := TStringList.Create;
//
// Fill the shadow values with invalid information to start with
//
    sShadowVoice1_Freq          := '';
    sShadowVoice1_FreqLo        := '';
    sShadowVoice1_FreqHi        := '';
    sShadowVoice1_PulseWidth    := '';
    sShadowVoice1_PulseWidthLo  := '';
    sShadowVoice1_PulseWidthHi  := '';
    sShadowVoice1_WaveForm      := '';
    sShadowVoice1_ADSR          := '';
    sShadowVoice1_AttackDecay   := '';
    sShadowVoice1_SustainRelease := '';
    sShadowVoice1_Volume        := '';

    sShadowVoice2_Freq          := '';
    sShadowVoice2_FreqLo        := '';
    sShadowVoice2_FreqHi        := '';
    sShadowVoice2_PulseWidth    := '';
    sShadowVoice3_PulseWidth    := '';
    sShadowVoice3_PulseWidthLo  := '';
    sShadowVoice2_WaveForm      := '';
    sShadowVoice2_ADSR          := '';
    sShadowVoice2_AttackDecay   := '';
    sShadowVoice2_SustainRelease := '';
    sShadowVoice2_Volume        := '';

    sShadowVoice3_Freq          := '';
    sShadowVoice3_FreqLo        := '';
    sShadowVoice3_FreqHi        := '';
    sShadowVoice3_PulseWidth    := '';
    sShadowVoice3_PulseWidth    := '';
    sShadowVoice3_PulseWidthLo  := '';
    sShadowVoice3_WaveForm      := '';
    sShadowVoice3_ADSR          := '';
    sShadowVoice3_AttackDecay   := '';
    sShadowVoice3_SustainRelease := '';
    sShadowVoice3_Volume        := '';

    sShadowVoice1MusicLineString   := '';
    sShadowVoice2MusicLineString   := '';
    sShadowVoice3MusicLineString   := '';

    sShadowDelayInterval        := '';
    sPreviousDelayInterval	    := '';
    sPreviousTime := '00:00.00';
    sCurrentTime := '00:00.00';
    iByteCount := 0;                                        // Amount of bytes in the CDATA table
    tFirstLine := True;                                     // Flag the first line as true so the initial CDATA statement can be placed
    iInsertionCount := 0;
    //iPreviousTime := 0;
//
// Scan the music data and add the results to a stringlist for later saving
//
    AddLine('''');
    AddLine(''' Tune Data for Amicus18 Simple SID player');
    AddLine('''');
    AddLine('TuneTable:');                                          // Add a generic Table name to the stringlist as a label
    AddLine('T_' + LeftStr(gTuneFileNameOnly,28) + ':');            // Add the tune's name to the stringlist as a label
    Repeat
        tCurrentTimeLoaded := False;                                //
        sVoice1 := '';                                              // Reset the string used for the register changes for Voice 1
        sVoice2 := '';                                              // Reset the string used for the register changes for Voice 2
        sVoice3 := '';                                              // Reset the string used for the register changes for Voice 3
        LineOfText := UpperCase(gMusicInList[LineIndex]);           // Extract a line of text from the stringlist
        //
        // Format the line so it's easier to parse
        //
        LineOfText := AnsiReplaceStr(LineOfText, '__ ___', '_____'); // Remove the space
        LineOfText := AnsiReplaceStr(LineOfText, '  ', ' ');        // Convert 2 spaces into 1 space
        LineOfText := AnsiReplaceStr(LineOfText, '  ', ' ');        // Convert 2 spaces into 1 space
        LineOfText := AnsiReplaceStr(LineOfText, '  ', ' ');        // Convert 2 spaces into 1 space
        LineOfText := AnsiReplaceStr(LineOfText, ' ', ',');         // Convert a space into a comma for parsing

        //Print(LineOfText);      // <<<<<<<<

        Parse.Commatext := LineOfText;                                      // Parse the line into the parse stringlist
        sTime := Parse[0];                                                  // Exstract the time from the line
        //
        // Extract Voice 1 values
        //
        iVoice1_Freq         := StrToInt(Parse[2]);                         // Convert 16-bit Decimal string to Int
        iVoice1_Freq         := iVoice1_Freq * cFrequencyMultiplier;        // Increase frequency of tone
        sVoice1_Freq         := IntToHex(iVoice1_Freq,4);                   // 16-bit Decimal to Hex
        sVoice1_FreqLo       := IntToHex((iVoice1_Freq AND 255),2);         // 8-bit Decimal to Hex
        sVoice1_FreqHi       := IntToHex((iVoice1_Freq SHR 8),2);           // 8-bit Decimal to Hex
        sVoice1_PulseWidth   := IntToHex(StrToInt(Parse[3]),4);             // 16-bit Decimal to Hex
        sVoice1_PulseWidthLo := IntToHex((StrToInt(Parse[3]) AND 255),2);   // 8-bit Decimal to Hex
        sVoice1_PulseWidthHi := IntToHex((StrToInt(Parse[3]) SHR 8),2);     // 8-bit Decimal to Hex
        sVoice1_WaveForm     := Parse[4];                                   // 8-bit Hex Value
        sVoice1_ADSR         := Parse[5];                                   // 16-bit Hex Value
        sVoice1_AttackDecay    := IntToHex((StrToInt('$' + Parse[5]) SHR 8),2);   // 8-bit Decimal to Hex (High Byte of ADSR)
        sVoice1_SustainRelease := IntToHex((StrToInt('$' + Parse[5]) AND 255),2); // 8-bit Decimal to Hex (Low Byte of ADSR)
        sVoice1_Volume       := Parse[6];                                   // 8-bit Hex Value
        If(sVoice1_Volume = '00') Then                                      // Is the volume parameter 0?
            begin                                                           // Yes. So...
                sVoice1_FreqLo := '00';                                     // Clear the low byte of the Frequency
                sVoice1_FreqHi := '00';                                     // Clear the high byte of the Frequency
                sVoice1_Freq := '0000';                                     // Clear all the Frequency
            end;
        //
        // Extract Voice 2 values
        //
        iVoice2_Freq         := StrToInt(Parse[8]);                         // Convert 16-bit Decimal string to Int
        iVoice2_Freq         := iVoice2_Freq * cFrequencyMultiplier;        // Increase frequency of tone
        sVoice2_Freq         := IntToHex(iVoice2_Freq,4);                   // 16-bit Decimal to Hex
        sVoice2_FreqLo       := IntToHex((iVoice2_Freq AND 255),2);         // 8-bit Decimal to Hex
        sVoice2_FreqHi       := IntToHex((iVoice2_Freq SHR 8),2);           // 8-bit Decimal to Hex
        sVoice2_PulseWidth   := IntToHex(StrToInt(Parse[9]),4);             // Decimal to Hex
        sVoice2_PulseWidthLo := IntToHex((StrToInt(Parse[9]) AND 255),2);   // 8-bit Decimal to Hex
        sVoice2_PulseWidthHi := IntToHex((StrToInt(Parse[9]) SHR 8),2);     // 8-bit Decimal to Hex
        sVoice2_WaveForm     := Parse[10];                                  // 8-bit Hex Value
        sVoice2_ADSR         := Parse[11];                                  // 16-bit Hex Value
        sVoice2_AttackDecay    := IntToHex((StrToInt('$' + Parse[11]) SHR 8),2);  // 8-bit Decimal to Hex (High Byte of ADSR)
        sVoice2_SustainRelease := IntToHex((StrToInt('$' + Parse[11]) AND 255),2);// 8-bit Decimal to Hex (Low Byte of ADSR)
        sVoice2_Volume       := Parse[12];                                  // 8-bit Hex Value
        If(sVoice2_Volume = '00') Then                                      // Is the volume parameter 0?
            begin                                                           // Yes. So...
                sVoice2_FreqLo := '00';                                     // Clear the low byte of the Frequency
                sVoice2_FreqHi := '00';                                     // Clear the high byte of the Frequency
                sVoice2_Freq := '0000';                                     // Clear all the Frequency
            end;
        //
        // Extract Voice 3 values
        //
        iVoice3_Freq         := StrToInt(Parse[14]);                        // Convert 16-bit Decimal string to Int
        iVoice3_Freq         := iVoice3_Freq * cFrequencyMultiplier;        // Increase frequency of tone
        sVoice3_Freq         := IntToHex(iVoice3_Freq,4);                   // 16-bit Decimal to Hex
        sVoice3_FreqLo       := IntToHex((iVoice3_Freq AND 255),2);         // 8-bit Decimal to Hex
        sVoice3_FreqHi       := IntToHex((iVoice3_Freq SHR 8),2);           // 8-bit Decimal to Hex
        sVoice3_PulseWidth   := IntToHex(StrToInt(Parse[15]),4);            // Decimal to Hex
        sVoice3_PulseWidthLo := IntToHex((StrToInt(Parse[15]) AND 255),2);  // 8-bit Decimal to Hex
        sVoice3_PulseWidthHi := IntToHex((StrToInt(Parse[15]) SHR 8),2);    // 8-bit Decimal to Hex
        sVoice3_WaveForm     := Parse[16];                                  // 8-bit Hex Value
        sVoice3_ADSR         := Parse[17];                                  // 16-bit Hex Value
        sVoice3_AttackDecay    := IntToHex((StrToInt('$' + Parse[17]) SHR 8),2);  // 8-bit Decimal to Hex (High Byte of ADSR)
        sVoice3_SustainRelease := IntToHex((StrToInt('$' + Parse[17]) AND 255),2);// 8-bit Decimal to Hex (Low Byte of ADSR)
        sVoice3_Volume       := Parse[18];                                  // 8-bit Hex Value
        If(sVoice3_Volume = '00') Then                                      // Is the volume parameter 0?
            begin                                                           // Yes. So...
                sVoice3_FreqLo := '00';                                     // Clear the low byte of the Frequency
                sVoice3_FreqHi := '00';                                     // Clear the high byte of the Frequency
                sVoice3_Freq := '0000';                                     // Clear all the Frequency
            end;

        sVoice1MusicLine := sVoice1_Freq + sVoice1_PulseWidth + sVoice1_WaveForm + sVoice1_ADSR;
        sVoice2MusicLine := sVoice2_Freq + sVoice2_PulseWidth + sVoice2_WaveForm + sVoice2_ADSR;
        sVoice3MusicLine := sVoice3_Freq + sVoice3_PulseWidth + sVoice3_WaveForm + sVoice3_ADSR;

        tVoice1_Used := False;                                      // Reset the Voice 1 used flag
        tVoice2_Used := False;                                      // Reset the Voice 2 used flag
        tVoice3_Used := False;                                      // Reset the Voice 3 used flag
//
// Voice 1
//
        If(ChooseVoice_1.Checked = true) AND (sVoice1MusicLine <> sShadowVoice1MusicLineString) Then   // Is the line of music for Voice 1 the same as the previous Voice 1 line of music ?
            begin                                                   // No. So...
                //
                // Check if individual registers require changing
                //
                If(sVoice1_Freq <> sShadowVoice1_Freq) Then                     // Is the current Frequency value the same as the previous value?
                    begin                                                       // No. So...
                        //
                        // Check if the Low or High bytes of the frequency need to be changed
                        //
                        tVoice1_Used := False;
                        If(sVoice1_FreqLo <> sShadowVoice1_FreqLo) and (sVoice1_FreqHi = sShadowVoice1_FreqHi) Then    // Is the low byte the same and the high byte different as the previous high byte?
                            begin                                                   // No. So...
                                tVoice1_Used := True;                               // Signal that voice 1 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;               // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice1 := sVoice1 + '''dPrevVoice1_Freq($' + sShadowVoice1_Freq + ') , ' + #10;
                                //sVoice1 := sVoice1 + '      ''dVoice1_Freq($' + sVoice1_Freq + ') , ' + #10;
                                sVoice1 := sVoice1 + 'dVoice1_FreqLo($' + sVoice1_FreqLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else If(sVoice1_FreqLo = sShadowVoice1_FreqLo) and (sVoice1_FreqHi <> sShadowVoice1_FreqHi) Then    // Is the low byte different and the high byte the same as the previous high byte?
                            begin                                                   // No. So...
                                tVoice1_Used := True;                               // Signal that voice 1 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;              // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice1 := sVoice1 + '''dPrevVoice1_Freq($' + sShadowVoice1_Freq + ') , ' + #10;
                                //sVoice1 := sVoice1 + '      ''dVoice1_Freq($' + sVoice1_Freq + ') , ' + #10;
                                sVoice1 := sVoice1 + 'dVoice1_FreqHi($' + sVoice1_FreqHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else
                        	begin
                                tVoice1_Used := True;                               	// Signal that voice 1 register has been updated
                        		If(tCurrentTimeLoaded = False) Then                 	// Has there been a previous time logged?
                        			begin                                           	// No. So...
                            			sCurrentTime := sTime;                      	// Log the time on this line of music
                                		iInsertionCount := gLineCount;               	// Keep a record of this line position within the output stringlist
                                		tCurrentTimeLoaded := True                  	// Signal no more time values to be logged for this scan
                            		end;
                             	sVoice1 := sVoice1 + 'dVoice1_Freq($' + sVoice1_Freq + ') , ';
                        		iByteCount := iByteCount + 4;                       	// Update the table size counter
                            end;
                    end;
                If(sVoice1_PulseWidth <> sShadowVoice1_PulseWidth) Then             // Is the Pulse Width value the same as the previous value?
                    begin                                                           // No. So...
                        //
                        // Check if the Low or High bytes of the Pulse Width need to be changed
                        //
                        // **** tVoice1_Used := False;
                        If(sVoice1_PulseWidthLo <> sShadowVoice1_PulseWidthLo) and (sVoice1_PulseWidthHi = sShadowVoice1_PulseWidthHi) Then    // Is the low byte the same and the high byte different as the previous high byte?
                            begin                                                   // No. So...
                                tVoice1_Used := True;                               // Signal that voice 1 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;              // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice1 := sVoice1 + '''dPrevVoice1_Pwm($' + sShadowVoice1_PulseWidth + ') , ' + #10;
                                //sVoice1 := sVoice1 + '      ''dVoice1_Pwm($' + sVoice1_PulseWidth + ') , ' + #10;
                                sVoice1 := sVoice1 + 'dVoice1_PwmLo($' + sVoice1_PulseWidthLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else If(sVoice1_PulseWidthLo = sShadowVoice1_PulseWidthLo) and (sVoice1_PulseWidthHi <> sShadowVoice1_PulseWidthHi) Then    // Is the low byte different and the high byte the same as the previous high byte?
                            begin                                                   // No. So...
                                tVoice1_Used := True;                               // Signal that voice 1 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;              // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice1 := sVoice1 + '''dPrevVoice1_Pwm($' + sShadowVoice1_PulseWidth + ') , ' + #10;
                                //sVoice1 := sVoice1 + '      ''dVoice1_Pwm($' + sVoice1_PulseWidth + ') , ' + #10;
                                sVoice1 := sVoice1 + 'dVoice2_PwmHi($' + sVoice1_PulseWidthHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else
                        	begin
                                tVoice1_Used := True;                               // Signal that voice 1 register has been updated
                        		If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                        			begin                                           // No. So...
                            			sCurrentTime := sTime;                      // Log the time on this line of music
                                		iInsertionCount := gLineCount;              // Keep a record of this line position within the output stringlist
                                		tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                            		end;
                             	sVoice1 := sVoice1 + 'dVoice1_Pwm($' + sVoice1_PulseWidth + ') , ';
                        		iByteCount := iByteCount + 4;                       // Update the table size counter
                            end;
                        {
                        If(sVoice1_PulseWidthLo <> sShadowVoice1_PulseWidthLo) Then  // Is the low byte the same as the previous low byte?
                            begin
                                sVoice1 := sVoice1 + 'dVoice1_PwmLo($' + sVoice1_PulseWidthLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        If(sVoice1_PulseWidthHi <> sShadowVoice1_PulseWidthHi) Then // Is the high byte the same as the previous high byte?
                            begin
                                sVoice1 := sVoice1 + 'dVoice1_PwmHi($' + sVoice1_PulseWidthHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        }
                    end;
                If(sVoice1_WaveForm <> sShadowVoice1_WaveForm) Then                 // Is the Waveform value the same as the previous value?
                    begin                                                           // No. So...
                        //tVoice1_Used := True;                                     // Signal that voice 1 register has been updated
                        sVoice1 := sVoice1 + 'dVoice1_Wave($' + sVoice1_WaveForm + ') , ';
                        iByteCount := iByteCount + 2;                               // Update the table size counter
                    end;
                If(sVoice1_ADSR <> sShadowVoice1_ADSR) Then                         // Is the ADSR value the same as the previous value?
                    begin                                                           // No. So...
                        //
                        // Check if the Low or High bytes of the ADSR need to be changed. i.e. not the same as the previous values
                        //
                        // **** tVoice1_Used := False;
                        If(sVoice1_AttackDecay <> sShadowVoice1_AttackDecay) Then   // Is the byte the same as the previous byte?
                            begin
                                sVoice1 := sVoice1 + 'dVoice1_AttDec($' + sVoice1_AttackDecay + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        If(sVoice1_SustainRelease <> sShadowVoice1_SustainRelease) Then   // Is the byte the same as the previous byte?
                            begin
                                sVoice1 := sVoice1 + 'dVoice1_SusRel($' + sVoice1_SustainRelease + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                    end;
                //
                // Add a preceding cdata statement if this is the first line
                //
                If(tFirstLine = True) Then                                          // Is this the first line ?
                    begin                                                           // Yes. So...
                        AddLine('Cdata ' + sVoice1 + '_');                          // Place a preceding cdata statement before the Voice 1 information
                        tFirstLine := false;                                        // Signal that the first line has been passed
                    end
                Else                                                                // Otherwise...
                    begin
                        AddLine('      ' + sVoice1 + '_');                          // Just place the Voice 1 information
                    end;
            end;
//
// Voice 2
//
        If(ChooseVoice_2.Checked = true) AND (sVoice2MusicLine <> sShadowVoice2MusicLineString) Then   // Is the line of music for Voice 2 the same as the previous Voice 2 line of music ?
            begin                                                                   // No. So...
                //
                // Check if individual registers require changing
                //
                If(sVoice2_Freq <> sShadowVoice2_Freq) Then                         // Is the Frequency value the same as the previous value?
                    begin                                                           // No. So...
                        //
                        // Check if the Low or High bytes of the frequency need to be changed
                        //
                        // **** tVoice2_Used := False;
                        If(sVoice2_FreqLo <> sShadowVoice2_FreqLo) and (sVoice2_FreqHi = sShadowVoice2_FreqHi) Then    // Is the low byte the same and the high byte different as the previous high byte?
                            begin                                                   // No. So...
                                tVoice2_Used := True;                               // Signal that voice 2 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;               // Keep a record of this line position within the output stringlist
                                        //Print('CurrentTime: ' + sTime);
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice2 := sVoice2 + '''dPrevVoice2_Freq($' + sShadowVoice2_Freq + ') , ' + #10;
                                //sVoice2 := sVoice2 + '      ''dVoice2_Freq($' + sVoice2_Freq + ') , ' + #10;
                                sVoice2 := sVoice2 + 'dVoice2_FreqLo($' + sVoice2_FreqLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else If(sVoice2_FreqLo = sShadowVoice2_FreqLo) and (sVoice2_FreqHi <> sShadowVoice2_FreqHi) Then    // Is the low byte different and the high byte the same as the previous high byte?
                            begin                                                   // No. So...
                                tVoice2_Used := True;                               // Signal that voice 2 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;              // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice2 := sVoice2 + '''dPrevVoice2_Freq($' + sShadowVoice2_Freq + ') , ' + #10;
                                //sVoice2 := sVoice2 + '      ''dVoice2_Freq($' + sVoice2_Freq + ') , ' + #10;
                                sVoice2 := sVoice2 + 'dVoice2_FreqHi($' + sVoice2_FreqHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else
                        	begin
                            	tVoice2_Used := True;                               	// Signal that voice 2 register has been updated
                             	If(tCurrentTimeLoaded = False) Then                 	// Has there been a previous time logged?
                        			begin                                           	// No. So...
                            			sCurrentTime := sTime;                      	// Log the time on this line of music
                                		iInsertionCount := gLineCount;               	// Keep a record of this line position within the output stringlist
                                		tCurrentTimeLoaded := True                  	// Signal no more time values to be logged
                            		end;
                             	sVoice2 := sVoice2 + 'dVoice2_Freq($' + sVoice2_Freq + ') , ';
                        		iByteCount := iByteCount + 4;                       	// Update the table size counter
                            end;
                    end;

                If(sVoice2_PulseWidth <> sShadowVoice2_PulseWidth) Then             // Is the Pulse Width value the same as the previous value?
                    begin                                                           // No. So...
                        //
                        // Check if the Low or High bytes of the Pulse Width need to be changed
                        //
                        // **** tVoice2_Used := False;
                        If(sVoice2_PulseWidthLo <> sShadowVoice2_PulseWidthLo) and (sVoice2_PulseWidthHi = sShadowVoice2_PulseWidthHi) Then    // Is the low byte the same and the high byte different as the previous high byte?
                            begin                                                   // No. So...
                                tVoice2_Used := True;                               // Signal that voice 2 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;               // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice2 := sVoice2 + '''dPrevVoice2_Pwm($' + sShadowVoice2_PulseWidth + ') , ' + #10;
                                //sVoice2 := sVoice2 + '      ''dVoice2_Pwm($' + sVoice2_PulseWidth + ') , ' + #10;
                                sVoice2 := sVoice2 + 'dVoice2_PwmLo($' + sVoice2_PulseWidthLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else If(sVoice2_PulseWidthLo = sShadowVoice2_PulseWidthLo) and (sVoice2_PulseWidthHi <> sShadowVoice2_PulseWidthHi) Then    // Is the low byte different and the high byte the same as the previous high byte?
                            begin                                                   // No. So...
                                tVoice2_Used := True;                               // Signal that voice 2 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;              // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice2 := sVoice2 + '''dPrevVoice2_Pwm($' + sShadowVoice2_PulseWidth + ') , ' + #10;
                                //sVoice2 := sVoice2 + '      ''dVoice2_Pwm($' + sVoice2_PulseWidth + ') , ' + #10;
                                sVoice2 := sVoice2 + 'dVoice2_PwmHi($' + sVoice2_PulseWidthHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else
                        	begin
                                tVoice2_Used := True;                               	// Signal that voice 3 register has been updated
                        		If(tCurrentTimeLoaded = False) Then                 	// Has there been a previous time logged?
                        			begin                                           	// No. So...
                            			sCurrentTime := sTime;                      	// Log the time on this line of music
                                		iInsertionCount := gLineCount;               	// Keep a record of this line position within the output stringlist
                                		tCurrentTimeLoaded := True                  	// Signal no more time values to be logged
                            		end;
                             	sVoice2 := sVoice2 + 'dVoice2_Pwm($' + sVoice2_PulseWidth + ') , ';
                        		iByteCount := iByteCount + 4;                       	// Update the table size counter
                            end;
                        {
                        If(sVoice2_PulseWidthLo <> sShadowVoice2_PulseWidthLo) Then  // Is the low byte the same as the previous low byte?
                            begin
                                sVoice2 := sVoice2 + 'dVoice2_PwmLo($' + sVoice2_PulseWidthLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        If(sVoice2_PulseWidthHi <> sShadowVoice2_PulseWidthHi) Then // Is the high byte the same as the previous high byte?
                            begin
                                sVoice2 := sVoice2 + 'dVoice2_PwmHi($' + sVoice2_PulseWidthHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        }
                    end;
                If(sVoice2_WaveForm <> sShadowVoice2_WaveForm) Then                 // Is the Waveform value the same as the previous value?
                    begin                                                           // No. So...
                        sVoice2 := sVoice2 + 'dVoice2_Wave($' + sVoice2_WaveForm + ') , ';
                        iByteCount := iByteCount + 2;                               // Update the table size counter
                    end;
                If(sVoice2_ADSR <> sShadowVoice2_ADSR) Then                         // Is the ADSR value the same as the previous value?
                    begin                                                           // No. So...
                        //
                        // Check if the Low or High bytes of the ADSR need to be changed. i.e. not the same as the previous values
                        //
                        // **** tVoice2_Used := False;
                        If(sVoice2_AttackDecay <> sShadowVoice2_AttackDecay) Then   // Is the byte the same as the previous byte?
                            begin
                                sVoice2 := sVoice2 + 'dVoice2_AttDec($' + sVoice2_AttackDecay + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        If(sVoice2_SustainRelease <> sShadowVoice2_SustainRelease) Then   // Is the byte the same as the previous byte?
                            begin
                                sVoice2 := sVoice2 + 'dVoice2_SusRel($' + sVoice2_SustainRelease + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                    end;
                //
                // Add a preceding cdata statement if this is the first line
                //
                If(tFirstLine = True) Then                                          // Is this the first line ?
                    begin                                                           // Yes. So...
                        AddLine('Cdata ' + sVoice2 + '_');                          // Place a preceding cdata statement before the Voice 2 information
                        tFirstLine := false;                                        // Signal that the first line has been passed
                    end
                Else                                                                // Otherwise...
                    begin
                        AddLine('      ' + sVoice2 + '_');                          // Just place the Voice 2 information
                    end;
            end;
//
// Voice 3
//
        If(ChooseVoice_3.Checked = true) AND (sVoice3MusicLine <> sShadowVoice3MusicLineString) Then   // Is the line of music for Voice 3 the same as the previous Voice 3 line of music ?
            begin                                                                   // No. So...
                //
                // Check if individual registers require changing
                //
                If(sVoice3_Freq <> sShadowVoice3_Freq) Then                         // Is the Frequency value the same as the previous value?
                    begin                                                           // No. So...
                        //
                        // Check if the Low or High bytes of the frequency need to be changed
                        //
                        // **** tVoice3_Used := False;
                        If(sVoice3_FreqLo <> sShadowVoice3_FreqLo) and (sVoice3_FreqHi = sShadowVoice3_FreqHi) Then    // Is the low byte the same and the high byte different as the previous high byte?
                            begin                                                   // No. So...
                                tVoice3_Used := True;                               // Signal that voice 3 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;               // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice3 := sVoice3 + '''dPrevVoice3_Freq($' + sShadowVoice3_Freq + ') , ' + #10;
                                //sVoice3 := sVoice3 + '      ''dVoice3_Freq($' + sVoice3_Freq + ') , ' + #10;
                                sVoice3 := sVoice3 + 'dVoice3_FreqLo($' + sVoice3_FreqLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else If(sVoice3_FreqLo = sShadowVoice3_FreqLo) and (sVoice3_FreqHi <> sShadowVoice3_FreqHi) Then    // Is the low byte different and the high byte the same as the previous high byte?
                            begin                                                   // No. So...
                                tVoice3_Used := True;                               // Signal that voice 3 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;              // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice3 := sVoice3 + '''dPrevVoice3_Freq($' + sShadowVoice3_Freq + ') , ' + #10;
                                //sVoice3 := sVoice3 + '      ''dVoice3_Freq($' + sVoice3_Freq + ') , ' + #10;
                                sVoice3 := sVoice3 + 'dVoice3_FreqHi($' + sVoice3_FreqHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else
                        	begin
                                tVoice3_Used := True;                               	// Signal that voice 3 register has been updated
                        		If(tCurrentTimeLoaded = False) Then                 	// Has there been a previous time logged?
                        			begin                                           	// No. So...
                            			sCurrentTime := sTime;                      	// Log the time on this line of music
                                		iInsertionCount := gLineCount;               	// Keep a record of this line position within the output stringlist
                                		tCurrentTimeLoaded := True                  	// Signal no more time values to be logged
                            		end;
                             	sVoice3 := sVoice3 + 'dVoice3_Freq($' + sVoice3_Freq + ') , ';
                        		iByteCount := iByteCount + 4;                       	// Update the table size counter
                            end;
                    end;
                If(sVoice3_PulseWidth <> sShadowVoice3_PulseWidth) Then             // Is the Pulse Width value the same as the previous value?
                    begin                                                           // No. So...
                        //
                        // Check if the Low or High bytes of the Pulse Width need to be changed
                        //
                        // **** tVoice3_Used := False;
                        If(sVoice3_PulseWidthLo <> sShadowVoice3_PulseWidthLo) and (sVoice3_PulseWidthHi = sShadowVoice3_PulseWidthHi) Then    // Is the low byte the same and the high byte different as the previous high byte?
                            begin                                                   // No. So...
                                tVoice3_Used := True;                               // Signal that voice 3 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;               // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice3 := sVoice3 + '''dPrevVoice3_Pwm($' + sShadowVoice3_PulseWidth + ') , ' + #10;
                                //sVoice3 := sVoice3 + '      ''dVoice3_Pwm($' + sVoice3_PulseWidth + ') , ' + #10;
                                sVoice3 := sVoice3 + 'dVoice3_PwmLo($' + sVoice3_PulseWidthLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else If(sVoice3_PulseWidthLo = sShadowVoice3_PulseWidthLo) and (sVoice3_PulseWidthHi <> sShadowVoice3_PulseWidthHi) Then    // Is the low byte different and the high byte the same as the previous high byte?
                            begin                                                   // No. So...
                                tVoice3_Used := True;                               // Signal that voice 3 register has been updated
                                If(tCurrentTimeLoaded = False) Then                 // Has there been a previous time logged?
                                    begin                                           // No. So...
                                        sCurrentTime := sTime;                      // Log the time on this line of music
                                        iInsertionCount := gLineCount;              // Keep a record of this line position within the output stringlist
                                        tCurrentTimeLoaded := True                  // Signal no more time values to be logged
                                    end;
                                //sVoice3 := sVoice3 + '''dPrevVoice3_Pwm($' + sShadowVoice3_PulseWidth + ') , ' + #10;
                                //sVoice3 := sVoice3 + '      ''dVoice3_Pwm($' + sVoice3_PulseWidth + ') , ' + #10;
                                sVoice3 := sVoice3 + 'dVoice3_PwmHi($' + sVoice3_PulseWidthHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end
                        Else
                        	begin
                                tVoice3_Used := True;                               	// Signal that voice 3 register has been updated
                        		If(tCurrentTimeLoaded = False) Then                 	// Has there been a previous time logged?
                        			begin                                           	// No. So...
                            			sCurrentTime := sTime;                      	// Log the time on this line of music
                                		iInsertionCount := gLineCount;               	// Keep a record of this line position within the output stringlist
                                		tCurrentTimeLoaded := True                  	// Signal no more time values to be logged
                            		end;
                             	sVoice3 := sVoice3 + 'dVoice3_Pwm($' + sVoice3_PulseWidth + ') , ';
                        		iByteCount := iByteCount + 4;                       	// Update the table size counter
                            end;
                        {
                        If(sVoice3_PulseWidthLo <> sShadowVoice3_PulseWidthLo) Then  // Is the low byte the same as the previous low byte?
                            begin
                                sVoice3 := sVoice3 + 'dVoice3_PwmLo($' + sVoice3_PulseWidthLo + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        If(sVoice3_PulseWidthHi <> sShadowVoice3_PulseWidthHi) Then // Is the high byte the same as the previous high byte?
                            begin
                                sVoice3 := sVoice3 + 'dVoice3_PwmHi($' + sVoice3_PulseWidthHi + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        }
                    end;

                If(sVoice3_WaveForm <> sShadowVoice3_WaveForm) Then                 // Is the Waveform value the same as the previous value?
                    begin                                                           // No. So...
                        sVoice3 := sVoice3 + 'dVoice3_Wave($' + sVoice3_WaveForm + ') , ';
                        iByteCount := iByteCount + 2;                               // Update the table size counter
                    end;

                If(sVoice3_ADSR <> sShadowVoice3_ADSR) Then                         // Is the ADSR value the same as the previous value?
                    begin                                                           // No. So...
                        //
                        // Check if the Low or High bytes of the ADSR need to be changed. i.e. not the same as the previous values
                        //
                        // **** tVoice3_Used := False;
                        If(sVoice3_AttackDecay <> sShadowVoice3_AttackDecay) Then   // Is the byte the same as the previous byte?
                            begin
                                sVoice3 := sVoice3 + 'dVoice3_AttDec($' + sVoice3_AttackDecay + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                        If(sVoice3_SustainRelease <> sShadowVoice3_SustainRelease) Then   // Is the byte the same as the previous byte?
                            begin
                                sVoice3 := sVoice3 + 'dVoice3_SusRel($' + sVoice3_SustainRelease + ') , ';
                                iByteCount := iByteCount + 2;                       // Update the table size counter
                            end;
                    end;
                //
                // Add a preceding cdata statement if this is the first line
                //
                If(tFirstLine = True) Then                                          // Is this the first line ?
                    begin                                                           // Yes. So...
                        AddLine('Cdata ' + sVoice3 + '_');                          // Place a preceding cdata statement before the Voice 3 information
                        tFirstLine := false;                                        // Signal that the first line has been passed
                    end
                Else                                                                // Otherwise...
                    begin
                        AddLine('      ' + sVoice3 + '_');                          // Just place the Voice 3 information
                    end;
            end;
//
// Place a delay between notes.
// Each delay has the amount of voices that were used before it
//
        //If(sShadowDelayInterval <> sDelayInterval) Then
        //    begin
                tPlaceATick := True;
                If((ChooseVoice_3.Checked = false) OR (ChooseVoice_2.Checked = false) OR (ChooseVoice_1.Checked = false)) Then
                    begin
                        //If(tVoice1_Used = True) OR (tVoice2_Used = True) OR (tVoice3_Used = True) Then
                            sTickName := 'Tick123';
                        //Else
                         //   tPlaceATick := False;
                    end
                Else
                    begin
                        If(tVoice1_Used = True) AND (tVoice2_Used = True) AND (tVoice3_Used = True) Then
                            sTickName := 'Tick123'
                        Else If(tVoice1_Used = True) AND (tVoice2_Used = True) Then
                            sTickName := 'Tick12'
                        Else If(tVoice1_Used = True) AND (tVoice3_Used = True) Then
                            sTickName := 'Tick13'
                        Else If(tVoice2_Used = True) AND (tVoice3_Used = True) Then
                            sTickName := 'Tick23'
                        Else If(tVoice1_Used = True) Then
                            sTickName := 'Tick1'
                        Else If(tVoice2_Used = True) Then
                            sTickName := 'Tick2'
                        Else If(tVoice3_Used = True) Then
                            sTickName := 'Tick3'
                        Else
                            tPlaceATick := False;
                    end;

                If(tPlaceATick = True) Then                                             			// Should a delay command be placed ?
                    begin
                        //Print('PreviousTime: ' + sPreviousTime);
                        sDelayInterval := GetTimeDifference(sCurrentTime,sPreviousTime);

                        iTemp := StrToInt(sDelayInterval);                              			// Convert the Interval String to an integer
                        //iTemp := iTemp * 4;                                         // Slow down the music by four times
                        If(sDelayInterval <> sShadowDelayInterval) Then
                        	begin
                        		If(iTemp > 255) Then                                              			// Trap a delay greate than 8-bits
                            		begin
                                		TickList.Add('dW' + sTickName + '(' + sDelayInterval + ') , _'); 	// Add the delay to the tick stringlist
                                		AddLine('dW' + sTickName + '(' + sDelayInterval + ') , _');
                                		iByteCount := iByteCount + 4;                           			// Update the table size counter
                            		end
                        		Else
                        			begin
                            			TickList.Add('d' + sTickName + '(' + sDelayInterval + ') , _'); 	// Add the delay to the tick stringlist
                                		AddLine('d' + sTickName + '(' + sDelayInterval + ') , _');
                                		iByteCount := iByteCount + 2;                           			// Update the table size counter
                            		end;
                        	end
                        Else
                        	begin
                				TickList.Add('dRep' + sTickName + '(' + sDelayInterval + ') , _'); 	// Add the delay to the tick stringlist
                               	AddLine('dRep' + sTickName + '(' + sDelayInterval + ') , _');
                        		iByteCount := iByteCount + 1;                           			// Update the table size counter
                            end;
                    end;
//
// Copy values into shadow variables for comparison within the parser
//
        sPreviousTime               := sCurrentTime;
        sShadowVoice1_Freq          := sVoice1_Freq;
        sShadowVoice1_FreqLo        := sVoice1_FreqLo;
        sShadowVoice1_FreqHi        := sVoice1_FreqHi;
        sShadowVoice1_PulseWidth    := sVoice1_PulseWidth;
        sShadowVoice1_PulseWidthLo  := sVoice1_PulseWidthLo;
        sShadowVoice1_PulseWidthHi  := sVoice1_PulseWidthHi;
        sShadowVoice1_WaveForm      := sVoice1_WaveForm;
        sShadowVoice1_ADSR          := sVoice1_ADSR;
        sShadowVoice1_AttackDecay   := sVoice1_AttackDecay;
        sShadowVoice1_SustainRelease := sVoice1_SustainRelease;
        sShadowVoice1_Volume        := sVoice1_Volume;

        sShadowVoice2_Freq          := sVoice2_Freq;
        sShadowVoice2_FreqLo        := sVoice2_FreqLo;
        sShadowVoice2_FreqHi        := sVoice2_FreqHi;
        sShadowVoice2_PulseWidth    := sVoice2_PulseWidth;
        sShadowVoice2_PulseWidthLo  := sVoice2_PulseWidthLo;
        sShadowVoice2_PulseWidthHi  := sVoice2_PulseWidthHi;
        sShadowVoice2_WaveForm      := sVoice2_WaveForm;
        sShadowVoice2_ADSR          := sVoice2_ADSR;
        sShadowVoice2_AttackDecay   := sVoice2_AttackDecay;
        sShadowVoice2_SustainRelease := sVoice2_SustainRelease;
        sShadowVoice2_Volume        := sVoice2_Volume;

        sShadowVoice3_Freq          := sVoice3_Freq;
        sShadowVoice3_FreqLo        := sVoice3_FreqLo;
        sShadowVoice3_FreqHi        := sVoice3_FreqHi;
        sShadowVoice3_PulseWidth    := sVoice3_PulseWidth;
        sShadowVoice3_PulseWidthLo  := sVoice3_PulseWidthLo;
        sShadowVoice3_PulseWidthHi  := sVoice3_PulseWidthHi;
        sShadowVoice3_WaveForm      := sVoice3_WaveForm;
        sShadowVoice3_ADSR          := sVoice3_ADSR;
        sShadowVoice3_AttackDecay   := sVoice3_AttackDecay;
        sShadowVoice3_SustainRelease := sVoice3_SustainRelease;
        sShadowVoice3_Volume        := sVoice3_Volume;

        sShadowVoice1MusicLineString := sVoice1MusicLine;
        sShadowVoice2MusicLineString := sVoice2MusicLine;
        sShadowVoice3MusicLineString := sVoice3MusicLine;

        //sPreviousDelayInterval		:= sDelayInterval - sShadowDelayInterval;
        sShadowDelayInterval        := sDelayInterval;
//
// Pointless going over the amount that the microcontroller can hold
//
        If (iByteCount >= 29000) Then Break;
        
        Inc(LineIndex);
    Until(LineIndex >= gMusicInList.Count) OR (gCloseButtonPressed = True);
//
// Add the finished command
//
    AddLine('      dFinish()');
    AddLine(''' Byte Count = ' + IntToStr(iByteCount));
//
// Shift the Delay intervals up one position within the stringlist
//
	gMusicInList.Clear();
    iTickIndex := 1;
    LineIndex := 0;
    Repeat
        LineOfText := gMusicOutList[LineIndex];

        If(AnsiPos('Tick',LineOfText) > 0) Then
        	begin
                Try
                	gMusicInList.Add('     ' + TickList[iTickIndex]);
                    Inc(iTickIndex);
                    //Print('     ' + TickList[iTickIndex]);
                Except
                End;
            end
        Else
            begin
           		gMusicInList.Add(LineOfText);
            	//Print(LineOfText);
            end;
    	Inc(LineIndex);
    Until(LineIndex >= gMusicOutList.Count) OR (gCloseButtonPressed = True);

    gMusicOutList.Clear();
    gMusicOutList.AddStrings(gMusicInList);
//
// Save the converted file
//
    DeleteFile(gConvertedMusicDirectory + ChangeFileExt(gTuneFileNameOnly,'.inc'));                 // Delete the original file
    gMusicOutList.SaveToFile(gConvertedMusicDirectory + ChangeFileExt(gTuneFileNameOnly,'.inc'));   // Save the new file

	Print('Complete. ' + IntToStr(iByteCount) + ' Bytes in Table');
//
//  Display the converted cdata table
//
{
    LineIndex := 0;
    Repeat
        Print(gMusicOutList[LineIndex]);
    	Inc(LineIndex);
    Until(LineIndex >= gMusicOutList.Count) OR (gCloseButtonPressed = True);
}
    TickList.Free;
    Parse.Free;
end;
//----------------------------------------------------------------------------
// Convert the time string into an interval
// Read the current time and the next time and subtract one from the other to give a time interval
// Returns the converted delay interval as a string
function TMainForm.ConvertTimeToInterval(pList: TStrings; pLine: Integer): String;
var
    TimeParse: TStringList;
    sCurrentLineOfText, sNextLineOfText: String;
    sCurrentTime, sNextTime: String;
    iCurrentMinutes, iCurrentSeconds, iCurrentCentiSeconds: Integer;
    iNextMinutes, iNextSeconds, iNextCentiSeconds: Integer;
    iTimeIntervalReturn: Integer;

    iCurrentTime, iNextTime: TDateTime;
    //MyCurrentTime: TDateTime;
    //MyNextTime: TDateTime;
begin
    If(pLine = pList.Count - 1) Then                                // Are we at the second last line of teh stringlist?
        begin                                                       // Yes. So...
            Result := 'End';                                        // Return a  result of "End"
            Exit;                                                   // Exit the routine
        end;
    TimeParse := TStringList.Create;                                // Create a stringlist to parse the line into
Try
    sCurrentLineOfText := pList[pLine];                             // Extract the current line of text from the stringlist passed
    sNextLineOfText := pList[pLine + 1];                            // Extract the next line of text from the stringlist passed

    sCurrentTime := AnsiReplaceStr(sCurrentLineOfText, ':', ',');   // Convert a colon into a comma for parsing the time value
    sCurrentTime := AnsiReplaceStr(sCurrentTime, '.', ',');         // Convert a period into a comma for parsing the time value
    sNextTime := AnsiReplaceStr(sNextLineOfText, ':', ',');         // Convert a colon into a comma for parsing the time value
    sNextTime := AnsiReplaceStr(sNextTime, '.', ',');               // Convert a period into a comma for parsing the time value
//
// Parse the current line and extract the current time
// This will deliberately cause an exception when there is no next line
//
    TimeParse.Commatext := sCurrentTime;
    iCurrentMinutes := StrToInt(TimeParse[0]);
    iCurrentSeconds := StrToInt(TimeParse[1]);
    iCurrentCentiSeconds := StrToInt(TimeParse[2]);
//
// Encode the current time into DateTime Format. Leave Hours as 0. Hour, Minute, Second, Millisecond (0 to 999)
//
    iCurrentTime := EncodeTime(0, iCurrentMinutes, iCurrentSeconds, iCurrentCentiSeconds * 10);
//
// Parse the next line and extract the next time
//
    TimeParse.Commatext := sNextTime;
    iNextMinutes := StrToInt(TimeParse[0]);
    iNextSeconds := StrToInt(TimeParse[1]);
    iNextCentiSeconds := StrToInt(TimeParse[2]);
//
// Encode the next time into DateTime Format. Leave Hours as 0. Hour, Minute, Second, Millisecond (0 to 999)
//
    iNextTime := EncodeTime(0, iNextMinutes, iNextSeconds, iNextCentiSeconds * 10);
//
// Find the difference between the next time and teh current time
// This will be the time interval to delay
//
    iTimeIntervalReturn := MilliSecondsBetween(iNextTime, iCurrentTime);
    //Print(IntToStr(iTimeIntervalReturn));

    TimeParse.Free;                                                             // Free the stringlist
    gInterval := iTimeIntervalReturn;                                           // Keep a record of this delay value
    Result := IntToStr(iTimeIntervalReturn);
Except
    //Print('Exception in ConvertTimeToInterval');
    TimeParse.Free;
    Result := 'End';
End;
end;
//----------------------------------------------------------------------------
// Convert the time strings into an interval
// Read the current time and the next time and subtract one from the other to give a time interval
// PtimeNaw and pTimeThen are in the raw format read from the music file
// Returns the converted delay interval as a string
function TMainForm.GetTimeDifference(pTimeNow: String; pTimeThen: String): String;
var
    TimeParse: TStringList;
    sCurrentTime, sPreviousTime: String;
    iCurrentMinutes, iCurrentSeconds, iCurrentCentiSeconds: Integer;
    iPreviousMinutes, iPreviousSeconds, iPreviousCentiSeconds: Integer;
    iTimeIntervalReturn: Integer;

    iCurrentTime, iPreviousTime: TDateTime;

begin
    TimeParse := TStringList.Create;                                // Create a stringlist to parse the line into
Try
    sCurrentTime := AnsiReplaceStr(pTimeNow, ':', ',');             // Convert a colon into a comma for parsing the time value
    sCurrentTime := AnsiReplaceStr(sCurrentTime, '.', ',');         // Convert a period into a comma for parsing the time value
    sPreviousTime := AnsiReplaceStr(pTimeThen, ':', ',');           // Convert a colon into a comma for parsing the time value
    sPreviousTime := AnsiReplaceStr(sPreviousTime, '.', ',');       // Convert a period into a comma for parsing the time value
//
// Parse the current time  string and extract the time
//
    TimeParse.Commatext := sCurrentTime;
    iCurrentMinutes := StrToInt(TimeParse[0]);
    iCurrentSeconds := StrToInt(TimeParse[1]);
    iCurrentCentiSeconds := StrToInt(TimeParse[2]);
//
// Encode the current time into DateTime Format. Leave Hours as 0. Hour, Minute, Second, Millisecond (0 to 999)
//
    iCurrentTime := EncodeTime(0, iCurrentMinutes, iCurrentSeconds, iCurrentCentiSeconds * 10);
//
// Parse the previous time and extract the time
//
    TimeParse.Commatext := sPreviousTime;
    iPreviousMinutes := StrToInt(TimeParse[0]);
    iPreviousSeconds := StrToInt(TimeParse[1]);
    iPreviousCentiSeconds := StrToInt(TimeParse[2]);
//
// Encode the next time into DateTime Format. Leave Hours as 0. Hour, Minute, Second, Millisecond (0 to 999)
//
    iPreviousTime := EncodeTime(0, iPreviousMinutes, iPreviousSeconds, iPreviousCentiSeconds * 10);
//
// Find the difference between the previous time and the current time
// This will be the time interval to delay
//
    iTimeIntervalReturn := MilliSecondsBetween(iCurrentTime,iPreviousTime);
    //Print(IntToStr(iTimeIntervalReturn));

    TimeParse.Free;                                                             // Free the stringlist
    //gInterval := iTimeIntervalReturn;                                         // Keep a record of this delay value

    //If(iTimeIntervalReturn > 255) Then
    //	Print('TimeInterval: ' + IntToStr(iTimeIntervalReturn));

    Result := IntToStr(iTimeIntervalReturn);
Except
    //Print('Exception in GetTimeDifference');
    TimeParse.Free;
    Result := '0';
End;
end;
//----------------------------------------------------------------------------
// Display text on the memo
procedure TMainForm.AddLine(pLineOfText: String);
begin
    gMusicOutList.Add(pLineOfText);
    Inc(gLineCount);					// Increment the line counter for the outgoing stringlist
    //Memo1.Lines.Add(pLineOfText);
    //Application.ProcessMessages();
end;
//----------------------------------------------------------------------------
// Display text on the memo
procedure TMainForm.Print(pLineOfText: String);
begin
    Memo1.Lines.Add(pLineOfText);
    Application.ProcessMessages();
end;
//----------------------------------------------------------------------------
// Clear teh memo
procedure TMainForm.Cls();
begin
    Memo1.Clear();
    Application.ProcessMessages();
end;
//----------------------------------------------------------------------------
// Timer event. Used to close the program
procedure TMainForm.CloseTimerTimer(Sender: TObject);
begin
    Close();
end;

end.
