'
' Simple SID Register Player
'
' Play a sequence of notes and adjust registers from a data table located in an include file
'
' Written by Les Johnson 01-01-11
'
' Use the low pass filter circuit below to hear the sounds
'
'                          1K             1uF
'                          ___           +[/
' From PORTC.2(RC2) o-----|___|-----o-----[|----o To Amplifier
'                                   |     [\
'                                   |
'                                  ---
'                                  --- 10nF
'                                   |
'                                   |
'                                  ===
'                                  GND
'
    Declare Reminders = Off
    Include "Simple SID.inc"                        ' Load the synthesizer routines into the program
'
' These variables are used in the Player
'
    Dim wPlayTimerStore As Word
    Dim wPlayTimer As Word
    Dim wSoundIndex As Word
    Dim bRegNumber As Byte
    Dim bMnemonic As bRegNumber
    Dim wTickTimer As Word
'
' Interpreter Mnemonics
'
' Defines
'
' SID Control Register masks
'
$define cGateOff 0
$define cGate 1
$define cSync 2         ' Not implemented
$define cRingMod 4
$define cTest 8         ' Not implemented
$define cTriangle 16
$define cSawtooth 32
$define cRectangle 64
$define cNoise 128

$define SlowAttack   $B0
$define FastAttack   $00
$define SlowDecay    $0A
$define FastDecay    $00
$define SustainQuiet $00
$define SustainNorm  $A0
$define SustainLoud  $F0
$define SlowRelease  $09
$define FastRelease  $00

$define dVoice1_GateOn() rVoice1_Control, $01
$define dVoice2_GateOn() rVoice2_Control, $01
$define dVoice3_GateOn() rVoice3_Control, $01

$define dVoice1_GateOff() rVoice1_Control, $0
$define dVoice2_GateOff() rVoice2_Control, $0
$define dVoice3_GateOff() rVoice3_Control, $0
'
' Alter 16-bit Voice Frequencies.
'
$define dVoice1_Freq(pNoteValue) rVoice1_FreqLo, pNoteValue & 255 , rVoice1_FreqHi, pNoteValue >> 8
$define dVoice2_Freq(pNoteValue) rVoice2_FreqLo, pNoteValue & 255 , rVoice2_FreqHi, pNoteValue >> 8
$define dVoice3_Freq(pNoteValue) rVoice3_FreqLo, pNoteValue & 255 , rVoice3_FreqHi, pNoteValue >> 8
'
' Alter 16-bit Voice PWM.
'
$define dVoice1_Pwm(pPWMValue) rVoice1_PulseWidthLo, pPWMValue & 255 , rVoice1_PulseWidthHi, pPWMValue >> 8
$define dVoice2_Pwm(pPWMValue) rVoice2_PulseWidthLo, pPWMValue & 255 , rVoice2_PulseWidthHi, pPWMValue >> 8
$define dVoice3_Pwm(pPWMValue) rVoice3_PulseWidthLo, pPWMValue & 255 , rVoice3_PulseWidthHi, pPWMValue >> 8
'
' Alter 8-bit Voice Attack Decay
'
$define dVoice1_AttDec(pAR)  rVoice1_AttackDecay, pAR
$define dVoice2_AttDec(pAR)  rVoice2_AttackDecay, pAR
$define dVoice3_AttDec(pAR)  rVoice3_AttackDecay, pAR
'
' Alter 8-bit Voice Sustain Release
'
$define dVoice1_SusRel(pSR)  rVoice1_SustainRelease, pSR
$define dVoice2_SusRel(pSR)  rVoice2_SustainRelease, pSR
$define dVoice3_SusRel(pSR)  rVoice3_SustainRelease, pSR
'
' Alter 8-bit Voice Waveform.
'
$define dVoice1_Wave(pCnt) rVoice1_Control, pCnt
$define dVoice2_Wave(pCnt) rVoice2_Control, pCnt
$define dVoice3_Wave(pCnt) rVoice3_Control, pCnt
'
' 16-bit delays interpreted by the player
' Bit-7 indicates a 16-bit delay, while the first 3-bits are the amount of voices to gate
'
$define dWTick1(pDly) $81 , pDly
$define dWTick2(pDly) $82 , pDly
$define dWTick3(pDly) $84 , pDly
$define dWTick12(pDly) $83 , pDly
$define dWTick13(pDly) $85 , pDly
$define dWTick23(pDly) $86 , pDly
$define dWTick123(pDly) $87 , pDly
'
' 8-bit delays interpreted by the player
' Bit-6 indicates an 8-bit delay, while the first 3-bits are the amount of voices to gate
'
$define dTick1(pDly) $41 , pDly
$define dTick2(pDly) $42 , pDly
$define dTick3(pDly) $44 , pDly
$define dTick12(pDly) $43 , pDly
$define dTick13(pDly) $45 , pDly
$define dTick23(pDly) $46 , pDly
$define dTick123(pDly) $47 , pDly
'
' Repeat a delay
'
$define dRepTick1(pDmy) $48
$define dRepTick2(pDmy) $49
$define dRepTick3(pDmy) $4A
$define dRepTick12(pDmy) $4B
$define dRepTick13(pDmy) $4C
$define dRepTick23(pDmy) $4D
$define dRepTick123(pDmy) $4E
'
' Alter 8-bit Low or High Voice Frequencies.
'
$define dVoice1_FreqLo(pNoteValue) $31 , pNoteValue
$define dVoice1_FreqHi(pNoteValue) $32 , pNoteValue
$define dVoice2_FreqLo(pNoteValue) $33 , pNoteValue
$define dVoice2_FreqHi(pNoteValue) $34 , pNoteValue
$define dVoice3_FreqLo(pNoteValue) $35 , pNoteValue
$define dVoice3_FreqHi(pNoteValue) $36 , pNoteValue
'
' Alter 8-bit Voice PWM.
'
$define dVoice1_PwmLo(pPWMValue) $37 , pPWMValue
$define dVoice1_PwmHi(pPWMValue) $38 , pPWMValue
$define dVoice2_PwmLo(pPWMValue) $39 , pPWMValue
$define dVoice2_PwmHi(pPWMValue) $3A , pPWMValue
$define dVoice3_PwmLo(pPWMValue) $3B , pPWMValue
$define dVoice3_PwmHi(pPWMValue) $3C , pPWMValue

$define cReplayVoice1 $F7
$define cReplayVoice1 $F8
$define cReplayVoice1 $F9

$define dFinish() $FF
'---------------------------------------------------------------------------------------------------
' Play a sequence of registers located in an include file
'
Main:
    wPlayTimer = 10
    While 1 = 1
        GoSub ADSR                                                                  ' Perform ADSR
        '
        ' Interpret the sequence of data held in the tune table
        '
        If wPlayTimer <= 0 Then                                                     ' Don't enter the Playr loop until the timer has counted down
            tFlag = true
            While tFlag = true                                                      ' Loop until tFlag is false. (triggered by a delay mnemonic)
                bMnemonic = CRead8 TuneTable[wSoundIndex]                           ' Read a mnemonic from the tune table
                Inc wSoundIndex                                                     ' Move up the table one position to the value after the mnemonic
                SID_bTemp = CRead8 TuneTable[wSoundIndex]                           ' Read the value from the tune table
                If bMnemonic < cNumRegisters Then                                   ' Is the value one of the Registers?
                    SetSidRegister(bRegNumber, SID_bTemp)                           ' Yes. So Update the appropriate SID register
                Else                                                                ' Otherwise it's another type of mnemonic
                    If bMnemonic = dFinish() Then                                   ' Stop mnemonic?
                        INTCOnbits_GIE = 0                                          ' Yes. So disable the sound interrupt
                        Stop                                                        ' Stop the program
                    ElseIf bMnemonic.7 = 1 Then                                     ' 16-bit delay?
                        wPlayTimer.LowByte = SID_bTemp                              ' Yes. So load the low byte of the delay value
                        Inc wSoundIndex                                             ' Move up the table one position for the high byte of the delay
                        wPlayTimer.HighByte = CRead8 TuneTable[wSoundIndex]         ' Read the high byte of the delay from the tune table
                        wPlayTimerStore =  wPlayTimer                               ' Keep a record of the delay value
                        wTickTimer = wPlayTimer / 2                                 ' Calculate the release time
                        bFlagByte = bFlagByte & %11111000                           ' Clear the voice gates
                        WREG =  bMnemonic & %00000111                               ' Mask out the voice gates from the delay mnemonic
                        bFlagByte =  bFlagByte | WREG                               ' OR in the new voice gates
                        If tVoice1_Trigger = true Then                              ' Signal Voice 1 ADSR attack?
                            SID_bVoice1_ControlReg.cBitGate = true                  ' Yes. So start the ADSR attack cycle for voice 1
                        EndIf
                        If tVoice2_Trigger = true Then                              ' Signal Voice 2 ADSR attack?
                            SID_bVoice2_ControlReg.cBitGate = true                  ' Yes. So start the ADSR attack cycle for voice 2
                        EndIf
                        If tVoice3_Trigger = true Then                              ' Signal Voice 3 ADSR attack?
                            SID_bVoice3_ControlReg.cBitGate = true                  ' Yes. So start the ADSR attack cycle for voice 3
                        EndIf
                        tFlag = false                                               ' Trigger an escape from the Playr loop
                    ElseIf bMnemonic.6 = 1 Then                                     ' 8-bit delay?
                        If bMnemonic = $48 Then                                     ' Repeat Delay with Voice 1 gate?
                            Dec wSoundIndex
                            wPlayTimer = wPlayTimerStore
                            tVoice1_Trigger = 1
                            tVoice2_Trigger = 0
                            tVoice3_Trigger = 0
                        ElseIf bMnemonic = $49 Then
                            Dec wSoundIndex
                            wPlayTimer = wPlayTimerStore
                            tVoice1_Trigger = 0
                            tVoice2_Trigger = 1
                            tVoice3_Trigger = 0
                        ElseIf bMnemonic = $4A Then
                            Dec wSoundIndex
                            wPlayTimer = wPlayTimerStore
                            tVoice1_Trigger = 0
                            tVoice2_Trigger = 0
                            tVoice3_Trigger = 1
                        ElseIf bMnemonic = $4B Then
                            Dec wSoundIndex
                            wPlayTimer = wPlayTimerStore
                            tVoice1_Trigger = 1
                            tVoice2_Trigger = 1
                            tVoice3_Trigger = 0
                        ElseIf bMnemonic = $4C Then
                            Dec wSoundIndex
                            wPlayTimer = wPlayTimerStore
                            tVoice1_Trigger = 1
                            tVoice2_Trigger = 0
                            tVoice3_Trigger = 1
                        ElseIf bMnemonic = $4D Then
                            Dec wSoundIndex
                            wPlayTimer = wPlayTimerStore
                            tVoice1_Trigger = 0
                            tVoice2_Trigger = 1
                            tVoice3_Trigger = 1
                        ElseIf bMnemonic = $4E Then
                            Dec wSoundIndex
                            wPlayTimer = wPlayTimerStore
                            tVoice1_Trigger = 1
                            tVoice2_Trigger = 1
                            tVoice3_Trigger = 1
                        Else
                            wPlayTimer = SID_bTemp                                  ' Yes. So load the delay value
                            wPlayTimerStore =  wPlayTimer                           ' Keep a record of the delay value
                            bFlagByte = bFlagByte & %11111000                       ' Clear the voice gates
                            WREG =  bMnemonic & %00000111                           ' Mask out the voice gates from the delay mnemonic
                            bFlagByte =  bFlagByte | WREG                           ' OR in the new voice gates
                        EndIf
                        wTickTimer = wPlayTimer / 2                                 ' Calculate the release time
                        If tVoice1_Trigger = true Then                              ' Signal Voice 1 ADSR attack?
                            SID_bVoice1_ControlReg.cBitGate = true                  ' Yes. So start the ADSR attack cycle for voice 1
                        EndIf
                        If tVoice2_Trigger = true Then                              ' Signal Voice 2 ADSR attack?
                            SID_bVoice2_ControlReg.cBitGate = true                  ' Yes. So start the ADSR attack cycle for voice 2
                        EndIf
                        If tVoice3_Trigger = true Then                              ' Signal Voice 3 ADSR attack?
                            SID_bVoice3_ControlReg.cBitGate = true                  ' Yes. So start the ADSR attack cycle for voice 3
                        EndIf
                        tFlag = false                                               ' Trigger an escape from the Playr loop
                    ElseIf bMnemonic.5 = 1 Then                                     ' A specific mnemonic
                        If bMnemonic = $31 Then                                     ' Voice 1 Low byte of frequency?
                            SID_wVoice1_Freq.LowByte = SID_bTemp
                            wFreqCoefficientV1 = SID_wVoice1_Freq
                        ElseIf bMnemonic = $32 Then                                 ' Voice 1 High byte of frequency?
                            SID_wVoice1_Freq.HighByte = SID_bTemp
                            wFreqCoefficientV1 = SID_wVoice1_Freq

                        ElseIf bMnemonic = $33 Then                                 ' Voice 2 Low byte of frequency?
                            SID_wVoice2_Freq.LowByte = SID_bTemp
                            wFreqCoefficientV2 = SID_wVoice2_Freq
                        ElseIf bMnemonic = $34 Then                                 ' Voice 2 High byte of frequency?
                            SID_wVoice2_Freq.HighByte = SID_bTemp
                            wFreqCoefficientV2 = SID_wVoice2_Freq

                        ElseIf bMnemonic = $35 Then                                 ' Voice 3 Low byte of frequency?
                            SID_wVoice3_Freq.LowByte = SID_bTemp
                            wFreqCoefficientV3 = SID_wVoice3_Freq
                        ElseIf bMnemonic = $36 Then                                 ' Voice 3 High byte of frequency?
                            SID_wVoice3_Freq.HighByte = SID_bTemp
                            wFreqCoefficientV3 = SID_wVoice3_Freq

                       ElseIf bMnemonic = $37 Then                                  ' Voice 1 Low byte of PWM?
                            SID_wVoice1_PW.LowByte = SID_bTemp
                            SetWaveForm(0)
                        ElseIf bMnemonic = $38 Then                                 ' Voice 1 High byte of PWM?
                            SID_wVoice1_PW.HighByte = SID_bTemp
                            SetWaveForm(0)

                        ElseIf bMnemonic = $39 Then                                 ' Voice 2 Low byte of PWM?
                            SID_wVoice2_PW.LowByte = SID_bTemp
                            SetWaveForm(1)
                        ElseIf bMnemonic = $3A Then                                 ' Voice 2 High byte of PWM?
                            SID_wVoice2_PW.HighByte = SID_bTemp
                            SetWaveForm(1)

                        ElseIf bMnemonic = $3B Then                                 ' Voice 3 Low byte of PWM?
                            SID_wVoice3_PW.LowByte = SID_bTemp
                            SetWaveForm(2)
                        ElseIf bMnemonic = $3C Then                                 ' Voice 3 High byte of PWM?
                            SID_wVoice3_PW.HighByte = SID_bTemp
                            SetWaveForm(2)
                        EndIf

                    EndIf
                EndIf
                Inc wSoundIndex                                                     ' Move up the table one position to the next mnemonic
            Wend                                                                    ' Close the Player loop
        Else
            Dec wPlayTimer                                                          ' Decrement the Play timer
            If wPlayTimer = 0 Then                                                  ' Have we reached the end of the delay?
                If tVoice1_Trigger = true Then                                      ' Yes. So Signal Voice 1 release?
                    SID_bVoice1_ControlReg.cBitGate = false                         ' Yes. SoADSR release cycle for voice 1
                EndIf
                If tVoice2_Trigger = true Then                                      ' Signal Voice 2 release?
                    SID_bVoice2_ControlReg.cBitGate = false                         ' Yes. So ADSR release cycle for voice 2
                EndIf
                If tVoice3_Trigger = true Then                                      ' Signal Voice 3 release?
                    SID_bVoice3_ControlReg.cBitGate = false                         ' Yes. So ADSR release cycle for voice 3
                EndIf
            EndIf
        EndIf
        DelayUS 550
    Wend                                                                            ' Close the main loop

'------------------------------------------------------------------------
' Load the tune data into the program
    Include "1942.inc"
    'Include "Ghosts_n_Goblins_subtune1.inc"
    'Include "Ghosts_n_Goblins_subtune2.inc"
    'Include "BombJack.inc"
    'Include "Rider_In_The_Sky.inc"

