'
' Experimental Sound Chip based upon the commodore SID device
'
' Written by Les Johnson 01-01-11
'
' Use the low pass filter circuit below to hear the sounds
'
'                          1K             1uF
'                          ___           +[/
' From PORTC.2(RC2) o-----|___|-----o-----[|----o To Amplifier
'                                   |     [\
'                                   |
'                                  ---
'                                  --- 10nF
'                                   |
'                                   |
'                                  ===
'                                  GND
'
' Note that the filters of the SID chip have not been emulated, but hopefully, these will be added at a later date.
'
    Declare Warnings = Off
    Declare Reminders = Off

    On_Hardware_Interrupt GoTo ISR_Timer0

    Declare Optimiser_Level = 3             ' Maximum optimisation
    Declare Dead_Code_Remove = On           ' Squeeze the code some more
$ifndef false
    $define false 0
$endif
$ifndef true
    $define true 1
$endif
'
' SID Registers, Names and Positions
'
$define rVoice1_FreqLo 0
$define rVoice1_FreqHi 1
$define rVoice1_PulseWidthLo 2
$define rVoice1_PulseWidthHi 3
$define rVoice1_Control 4
$define rVoice1_AttackDecay 5
$define rVoice1_SustainRelease 6

$define rVoice2_FreqLo 7
$define rVoice2_FreqHi 8
$define rVoice2_PulseWidthLo 9
$define rVoice2_PulseWidthHi 10
$define rVoice2_Control 11
$define rVoice2_AttackDecay 12
$define rVoice2_SustainRelease 13

$define rVoice3_FreqLo 14
$define rVoice3_FreqHi 15
$define rVoice3_PulseWidthLo 16
$define rVoice3_PulseWidthHi 17
$define rVoice3_Control 18
$define rVoice3_AttackDecay 19
$define rVoice3_SustainRelease 20

    Symbol cMaxLevel = 19000
    Symbol cSustainFactor  = cMaxLevel / 15
    Symbol cNumRegisters = 21

    Dim SID_wFSR0 As FSR0L.Word
    Dim SID_wFSR2 As FSR2L.Word
    Dim SID_wPROD As PRODL.Word
'
' Sid Registers
'
    Dim SID_wVoice1 As Word
    Dim SID_wVoice1_Freq As SID_wVoice1       ' Frequency: FreqLo/FreqHi
    Dim SID_wVoice1_PW As Word                ' PulseWidth: PW Lo/Hi
    Dim SID_bVoice1_ControlReg As Byte        ' Noise, Rectangle, SawTooth, Triangle, Test, RingMod, Sync, Gate
    Dim SID_bVoice1_AttackDecay As Byte       ' Bits 0-3 decay, Bit 4-7 attack
    Dim SID_bVoice1_SustainRelease As Byte    ' Bits 0-3 release, Bit 4-7 sustain

    Dim SID_wVoice2 As Word
    Dim SID_wVoice2_Freq As SID_wVoice2       ' Frequency: FreqLo/FreqHi
    Dim SID_wVoice2_PW As Word                ' PulseWidth: PW Lo/Hi
    Dim SID_bVoice2_ControlReg As Byte        ' Noise, Rectangle, SawTooth, Triangle, Test, RingMod, Sync, Gate
    Dim SID_bVoice2_AttackDecay As Byte       ' Bits 0-3 decay, Bits 4-7 attack
    Dim SID_bVoice2_SustainRelease As Byte    ' Bits 0-3 release, Bits 4-7 sustain

    Dim SID_wVoice3 As Word
    Dim SID_wVoice3_Freq As SID_wVoice3       ' Frequency: FreqLo/FreqHi
    Dim SID_wVoice3_PW As Word                ' PulseWidth: PW Lo/Hi
    Dim SID_bVoice3_ControlReg As Byte        ' Noise, Rectangle, SawTooth, Triangle, Test, RingMod, Sync, Gate
    Dim SID_bVoice3_AttackDecay As Byte       ' Bits 0-3 decay, Bits 4-7 attack
    Dim SID_bVoice3_SustainRelease As Byte    ' Bits 0-3 release, Bits 4-7 sustain

    Dim SID_bRegister[cNumRegisters] As Byte At SID_wVoice1
'
' SID Register Constants and Bits
'
    Symbol cBitGate =      0
    Symbol cBitSync =      1                 ' Not implemented
    Symbol cBitRingMod =   2                 ' Implemented
    Symbol cBitTEST =      3                 ' Not implemented
    Symbol cBitTriangle =  4
    Symbol cBitSawTooth =  5
    Symbol cBitRectangle = 6
    Symbol cBitNoise =     7

' SID Mode/Vol
    Symbol cVoice3Off = 128
    Symbol cBitVoice3Off = 7
'
' General variables
'
    Dim bVoiceNumber As Byte
    Dim SID_wTemp As Word                   ' Temporary word variable
    Dim SID_bTemp As Byte                   ' Temporary byte variable

    Dim SID_bIndex As bVoiceNumber
    Dim bIWtemp As SID_wTemp.Byte0

    Dim bSet_SidRegister_RegNum As PRODL    ' Parameter for SetSidRegister
    Dim bSet_SidRegister_Value As PRODH     ' Parameter for SetSidRegister
    Dim wWaveArrayPtr As FSR1L.Word

    Dim bFlagByte As Byte System            ' Used for flag bits (Must be in Access RAM)
    Dim tVoice1_Trigger As bFlagByte.0      ' Used in Player to signal if Voice 1 requires gating
    Dim tVoice2_Trigger As bFlagByte.1      ' Used in Player to signal if Voice 2 requires gating
    Dim tVoice3_Trigger As bFlagByte.2      ' Used in Player to signal if Voice 3 requires gating
    Dim tVoice1_AttackCycle As bFlagByte.3  ' Used for ADSR cycling
    Dim tVoice2_AttackCycle As bFlagByte.4  ' Used for ADSR cycling
    Dim tVoice3_AttackCycle As bFlagByte.5  ' Used for ADSR cycling
    Dim tIflag As bFlagByte.6               ' Used for toggling code within the interrupt handler
    Dim tFlag As bFlagByte.7                ' Used in Player for time scheduling
'
' Interrupt handler variables (Must be in Access RAM)
'
    Dim wFreqCoefficientV1 As Word System
    Dim wFreqCoefficientV2 As Word System
    Dim wFreqCoefficientV3 As Word System
    Dim wItemp As PRODL.Word                ' Temporary word variable aliased to SFRs PRODL\H
    Dim wPhase0 As Word System
    Dim wPhase1 As Word System
    Dim wPhase2 As Word System
    Dim wNoise  As Word System
    Dim wNoise8 As Word System
    Dim wSignal0 As Word System
    Dim wSignal1 As Word System
    Dim wSignal2 As Word System
    Dim wTempPhase As Word System
'
' Envelope variables used in ADSR
'
    Dim wTempAmp As Word
    Dim wAttackV1 As Word
    Dim wAttackV2 As Word
    Dim wAttackV3 As Word
    Dim wDecayV1 As Word
    Dim wDecayV2 As Word
    Dim wDecayV3 As Word
    Dim wReleaseV1 As Word
    Dim wReleaseV2 As Word
    Dim wReleaseV3 As Word
    Dim wSustainV1 As Word
    Dim wSustainV2 As Word
    Dim wSustainV3 As Word
    Dim wAmplitudeV1 As Word
    Dim wAmplitudeV2 As Word
    Dim wAmplitudeV3 As Word
    Dim bEnvelopeV1 As Byte
    Dim bEnvelopeV2 As Byte
    Dim bEnvelopeV3 As Byte
'
' Attack, Decay, Release Envelope timings
'
    Dim wAttackRate[16] As Word
    Dim wDecayReleaseRate[16] As Word
'
' Oscillator waveform arrays
'
    Dim bWave0[256] As Byte
    Dim bWave1[256] As Byte
    Dim bWave2[256] As Byte

    Dim wSavePROD As Word                            ' Saves/Restores the PRODL\H SFRs within the interrupt handler

$define DisableInterrupt() INTCONbits_GIE = 0
$define EnableInterrupt() INTCONbits_GIE = 1
'------------------------------------------------------------------
    GoTo SID_EmuMain                                    ' Jump over the subroutines
'------------------------------------------------------------------
' Timer0 Overflow interrupt routine
' Calculate waveforms and phases
' Set PWM output
'
' Notes     : Disturbs FSR2L\H registers, which are not context saved
'
ISR_Timer0:
    wSavePROD = SID_wPROD                               ' Save the contents of PRODL\H
    Btg tIflag
    If tIflag = 1 Then
        '
        ' Voice1
        '
        wTempPhase = wPhase0 + wFreqCoefficientV1
        If SID_bVoice1_ControlReg.cBitNoise = 1 Then
            wItemp = wTempPhase ^ wPhase0
            If wItemp.14 = 1 Then
                wSignal0 = wNoise8.Byte0 * bEnvelopeV1
            EndIf
        Else
            SID_wFSR2 = VarPtr bWave0                   ' \
            SID_wFSR2 = SID_wFSR2 + wPhase0.Byte1       '   Faster equivalent to: WREG = bWave0[wPhase0.Byte1]
            WREG = INDF2                                ' /
            If SID_bVoice1_ControlReg.cBitRingMod = 1 Then
                If wPhase2.15 = 1 Then                  ' Is wPhase2 a negative value ?
                    WREG = -WREG                        ' Yes. So make it positive
                EndIf
            EndIf
            wSignal0 = WREG * bEnvelopeV1
        EndIf
        wPhase0 = wTempPhase
        '
        ' Voice2
        '
        wTempPhase = wPhase1 + wFreqCoefficientV2
        If SID_bVoice2_ControlReg.cBitNoise = 1 Then
            wItemp = wTempPhase ^ wPhase1
            If wItemp.14 = 1 Then
                wSignal1 = wNoise8.Byte0 * bEnvelopeV2
            EndIf
        Else
            SID_wFSR2 = VarPtr bWave1                   ' \
            SID_wFSR2 = SID_wFSR2 + wPhase1.Byte1       '   Faster equivalent to: WREG = bWave1[wPhase1.Byte1]
            WREG = INDF2                                ' /
            If SID_bVoice2_ControlReg.cBitRingMod = 1 Then
                If wPhase0.15 = 1 Then                  ' Is wPhase0 a negative value ?
                    WREG = -WREG                        ' Yes. So make it positive
                EndIf
            EndIf
            wSignal1 = WREG * bEnvelopeV2
        EndIf
        wPhase1 = wTempPhase
        '
        ' Voice3
        '
        wTempPhase = wPhase2 + wFreqCoefficientV3
        If SID_bVoice3_ControlReg.cBitNoise = 1 Then
            wItemp = wTempPhase ^ wPhase2
            If wItemp.14 = 1 Then
                wSignal2 = wNoise8.Byte0 * bEnvelopeV3
            EndIf
        Else
            SID_wFSR2 = VarPtr bWave2                   ' \
            SID_wFSR2 = SID_wFSR2 + wPhase2.Byte1       '   Faster equivalent to: WREG = bWave2[wPhase2.Byte1]
            WREG = INDF2                                ' /
            If SID_bVoice3_ControlReg.cBitRingMod = 1 Then
                If wPhase1.15 = 1 Then                  ' Is wPhase1 a negative value ?
                    WREG = -WREG                        ' Yes. So make it positive
                EndIf
            EndIf
            wSignal2 = WREG * bEnvelopeV3
        EndIf
        wPhase2 = wTempPhase
    Else
        '
        ' Accumulate
        '
        wSignal0 = wSignal0 + wSignal1
        wSignal0 = wSignal0 + wSignal2
        wSignal0 = wSignal0 / 64                         ' Scale the value to avoid distortion
        '
        '  Output wSignal0 To PWM
        '
        CCP1CON = CCP1CON & %11001111               ' \
        WREG = wSignal0.Byte0 << 4                  '  |
        WREG = WREG & %00110000                     '  | Load bits 4 and 5 of CCP1CON with bits 0 and 1 of wSignal0
        CCP1CON = CCP1CON | WREG                    ' /
        CCPR1L = wSignal0 >> 2                      ' Load CCPR1L with the remaining 8 Most Significant bits shifted into place
        '
        ' Noise  generator
        '
        wSignal0 = wNoise
        wNoise = wNoise << 1
        wSignal0 = wSignal0 ^ wNoise
        If wSignal0.14 = 1 Then
            wNoise = wNoise | 1
        EndIf
        wNoise8 = wNoise >> 9
    EndIf

    SID_wPROD = wSavePROD                           ' Restore the contents of PRODL\H
    Clear INTCONbits_T0IF                           ' Clear the Timer0 overflow flag
    Retfie Fast                                     ' Exit the interrupt, restoring STATUS, BSR, and WREG

'------------------------------------------------------------------
' SetWaveForm
' Setup the waveforms for each voice
'
' Input     : bVoiceNumber holds the voice value: 0 = Voice 1, 1 = Voice 2, 2 = Voice 3
'           : SID_bVoicex_ControlReg holds the waveform type per voice
' Output    : Arrays bWave0, bWave1, and bWave1 hold the waveform values
' Notes     : None
'
$define SetWaveForm(pVN) bVoiceNumber = pVN : GoSub _SetWaveForm
_SetWaveForm:
    bIWtemp = 0
    If bVoiceNumber = 0 Then
        wWaveArrayPtr = VarPtr bWave0
        For SID_bIndex = 0 To 255
            If SID_bVoice1_ControlReg.cBitSawTooth = 1 Then
                POSTINC1 = bIWtemp
                If SID_bIndex.6 = 1 Then
                    bIWtemp = 0
                Else
                    Inc bIWtemp
                EndIf
            ElseIf SID_bVoice1_ControlReg.cBitTriangle = 1 Then
                If SID_bIndex.7 = 1 Then
                    WREG = bIWtemp
                    Dec bIWtemp
                Else
                    WREG = bIWtemp
                    Inc bIWtemp
                EndIf
                POSTINC1 = WREG
            ElseIf SID_bVoice1_ControlReg.cBitRectangle = 1 Then
                SID_bTemp = SID_wVoice1_PW >> 4
                If SID_bIndex > SID_bTemp Then
                    POSTINC1 = 64
                Else
                    POSTINC1 = 0
                EndIf
            EndIf
        Next
    ElseIf bVoiceNumber = 1 Then
        wWaveArrayPtr = VarPtr bWave1
        For SID_bIndex = 0 To 255
            If SID_bVoice2_ControlReg.cBitSawTooth = 1 Then
                POSTINC1 = bIWtemp
                If SID_bIndex.6 = 1 Then
                    bIWtemp = 0
                Else
                    Inc bIWtemp
                EndIf
            ElseIf SID_bVoice2_ControlReg.cBitTriangle = 1 Then
                If SID_bIndex.7 = 1 Then
                    WREG = bIWtemp
                    Dec bIWtemp
                Else
                    WREG = bIWtemp
                    Inc bIWtemp
                EndIf
                POSTINC1 = WREG
            ElseIf SID_bVoice2_ControlReg.cBitRectangle = 1 Then
                SID_bTemp = SID_wVoice2_PW >> 4
                If SID_bIndex > SID_bTemp Then
                    POSTINC1 = 64
                Else
                    POSTINC1 = 0
                EndIf
            EndIf
        Next
    ElseIf bVoiceNumber = 2 Then
        wWaveArrayPtr = VarPtr bWave2
        For SID_bIndex = 0 To 255
            If SID_bVoice3_ControlReg.cBitSawTooth = 1 Then
                POSTINC1 = bIWtemp
                If SID_bIndex.6 = 1 Then
                    bIWtemp = 0
                Else
                    Inc bIWtemp
                EndIf
            ElseIf SID_bVoice3_ControlReg.cBitTriangle = 1 Then
                If SID_bIndex.7 = 1 Then
                    WREG = bIWtemp
                    Dec bIWtemp
                Else
                    WREG = bIWtemp
                    Inc bIWtemp
                EndIf
                POSTINC1 = WREG
            ElseIf SID_bVoice3_ControlReg.cBitRectangle = 1 Then
                SID_bTemp = SID_wVoice3_PW >> 4
                If SID_bIndex > SID_bTemp Then
                    POSTINC1 = 64
                Else
                    POSTINC1 = 0
                EndIf
            EndIf
        Next
    EndIf
    Return
'------------------------------------------------------------------
' SetEnvelope
' Input     : bVoiceNumber holds the voice value: 0 = Voice 1, 1 = Voice 2, 2 = Voice 3
' Output    :
' Notes     : None
'
$define SetEnvelope(pVN) bVoiceNumber = pVN : GoSub _SetEnvelope
_SetEnvelope:
'
' These bits must be set seperately because they share the same byte as the interrupt toggle bit
'
    tVoice1_AttackCycle = 1
    tVoice2_AttackCycle = 1
    tVoice3_AttackCycle = 1

    If bVoiceNumber = 0 Then                                    ' Voice 1?
        WREG = SID_bVoice1_SustainRelease >> 4                  ' Extract the high nibble (Sustain rate)
        wSustainV1 = WREG * cSustainFactor

        WREG = SID_bVoice1_AttackDecay >> 4                     ' Extract the high nibble (Attack rate)
        wAttackV1 = cMaxLevel / wAttackRate[WREG]

        SID_bTemp = SID_bVoice1_AttackDecay & $0F               ' Extract the low nibble (Decay rate)
        SID_wTemp = wSustainV1 * cSustainFactor
        SID_wTemp = cMaxLevel - SID_wTemp
        wDecayV1 = SID_wTemp / wDecayReleaseRate[SID_bTemp]

        SID_bTemp = SID_bVoice1_SustainRelease & $0F            ' Extract the low nibble (Release rate)
        wReleaseV1 = wSustainV1 / wDecayReleaseRate[SID_bTemp]
    ElseIf bVoiceNumber = 1 Then                                ' Voice 2?
        WREG = SID_bVoice2_SustainRelease >> 4                  ' Extract the high nibble (Sustain rate)
        wSustainV2 = WREG * cSustainFactor

        WREG = SID_bVoice2_AttackDecay >> 4                     ' Extract the high nibble (Attack rate)
        wAttackV2 = cMaxLevel / wAttackRate[WREG]

        SID_bTemp = SID_bVoice2_AttackDecay & $0F               ' Extract the low nibble (Decay rate)
        SID_wTemp = wSustainV2 * cSustainFactor
        SID_wTemp = cMaxLevel - SID_wTemp
        wDecayV2 = SID_wTemp / wDecayReleaseRate[SID_bTemp]

        SID_bTemp = SID_bVoice2_SustainRelease & $0F            ' Extract the low nibble (Release rate)
        wReleaseV2 = wSustainV2 / wDecayReleaseRate[SID_bTemp]
    ElseIf bVoiceNumber = 2 Then                                ' Voice 3?
        WREG = SID_bVoice3_SustainRelease >> 4                  ' Extract the high nibble (Sustain rate)
        wSustainV3 = WREG * cSustainFactor

        WREG = SID_bVoice3_AttackDecay >> 4                     ' Extract the high nibble (Attack rate)
        wAttackV3 = cMaxLevel / wAttackRate[WREG]

        SID_bTemp = SID_bVoice3_AttackDecay & $0F               ' Extract the low nibble (Decay rate)
        SID_wTemp = wSustainV3 * cSustainFactor
        SID_wTemp = cMaxLevel - SID_wTemp
        wDecayV3 = SID_wTemp / wDecayReleaseRate[SID_bTemp]

        SID_bTemp = SID_bVoice3_SustainRelease & $0F            ' Extract the low nibble (Release rate)
        wReleaseV3 = wSustainV3 / wDecayReleaseRate[SID_bTemp]
    EndIf
    Return
'------------------------------------------------------------------
' SetSidRegister
' Set the registers of the virtual SID emulator
' Input     : bSet_SidRegister_RegNum holds the register of interest
'           : bSet_SidRegister_Value holds the value to place in the register
' Output    : None
' Notes     : None
'
$define SetSidRegister(pRegNum, pValue) '
    bSet_SidRegister_RegNum = pRegNum   '
    bSet_SidRegister_Value = pValue     '
    GoSub _SetSidRegister

_SetSidRegister:
'
' Place the value in the register
'
    SID_bRegister[bSet_SidRegister_RegNum] = bSet_SidRegister_Value
'
' Perform a specific task depending on which register was chosen
'
    Select bSet_SidRegister_RegNum
        '
        ' Voice 1
        '
        Case rVoice1_FreqHi                             ' High byte register of Voice 1?
            wFreqCoefficientV1 = SID_wVoice1_Freq
        Case rVoice1_PulseWidthHi, rVoice1_Control
            SetWaveForm(0)                              ' Point to Voice 1 for SetWaveForm subroutine
        Case rVoice1_AttackDecay, rVoice1_SustainRelease
            SetEnvelope(0)                              ' Point to Voice 1 for SetEnvelope subroutine
        '
        ' Voice 2
        '
        Case rVoice2_FreqHi                             ' High byte register of Voice 2?
            wFreqCoefficientV2 = SID_wVoice2_Freq
        Case rVoice2_PulseWidthHi, rVoice2_Control
            SetWaveForm(1)                              ' Point to Voice 2 for SetWaveForm subroutine
        Case rVoice2_AttackDecay, rVoice2_SustainRelease
            SetEnvelope(1)                              ' Point to Voice 2 for SetEnvelope subroutine
        '
        ' Voice 3
        '
        Case rVoice3_FreqHi                             ' High byte register of Voice 3?
            wFreqCoefficientV3 = SID_wVoice3_Freq
        Case rVoice3_PulseWidthHi, rVoice3_Control
            SetWaveForm(2)                              ' Point to Voice 3 for SetWaveForm subroutine
        Case rVoice3_AttackDecay, rVoice3_SustainRelease
            SetEnvelope(2)                              ' Point to Voice 3 for SetEnvelope subroutine
    EndSelect
    Return
'------------------------------------------------------------------------
' ADSR
' Calculate envelopes
'
' Input     :
' Output    :
' Notes     : If gate is true then the attack, decay, sustain cycle begins
'             If gate switches To false the Sound decays
ADSR:
'
' Voice 1 ADSR
'
    wTempAmp = wAmplitudeV1
    If SID_bVoice1_ControlReg.cBitGate = true Then  ' If gate set then attack,decay,sustain
        If tVoice1_AttackCycle = true Then          ' If attack cycle
            wTempAmp = wTempAmp + wAttackV1
            If wTempAmp > cMaxLevel Then
                wTempAmp = cMaxLevel
                tVoice1_AttackCycle = false         ' If level reached, then switch to decay
            EndIf
        Else                                        ' Decay cycle
            If wTempAmp > wSustainV1 Then
                wTempAmp = wTempAmp - wDecayV1
                If wTempAmp < wSustainV1 Then
                    wTempAmp = wSustainV1
                EndIf
            EndIf
        EndIf
    Else                                            ' If gate Flag is not set then release
        tVoice1_AttackCycle = true                  ' At next attack/decay cycle start with attack
        If wTempAmp.15 = 0 Then                     ' If wTempAmp > 0
        wTempAmp = wTempAmp - wReleaseV1
            If wTempAmp.15 = 1 Then                 ' If wTempAmp < 0
                wTempAmp = 0
            EndIf
        EndIf
    EndIf
    bEnvelopeV1 = wTempAmp.Byte1
    wAmplitudeV1 = wTempAmp
'
' Voice 2 ADSR
'
    wTempAmp = wAmplitudeV2
    If SID_bVoice2_ControlReg.cBitGate = true Then  ' If gate set then attack,decay,sustain
        If tVoice2_AttackCycle = true Then          ' If attack cycle
            wTempAmp = wTempAmp + wAttackV2
            If wTempAmp > cMaxLevel Then
                wTempAmp = cMaxLevel
                tVoice2_AttackCycle = false         ' If level reached, then switch to decay
            EndIf
        Else                                        ' Decay cycle
            If wTempAmp > wSustainV2 Then
                wTempAmp = wTempAmp - wDecayV2
                If wTempAmp < wSustainV2 Then
                    wTempAmp = wSustainV2
                EndIf
            EndIf
        EndIf
    Else                                            ' If gate Flag is not set then release
        tVoice2_AttackCycle = true                  ' At next attack/decay cycle start with attack
        If wTempAmp.15 = 0 Then                     ' If wTempAmp > 0
        wTempAmp = wTempAmp - wReleaseV2
        If wTempAmp.15 = 1 Then                     ' If wTempAmp < 0
            wTempAmp = 0
        EndIf
        EndIf
    EndIf
    bEnvelopeV2 = wTempAmp.Byte1
    wAmplitudeV2 = wTempAmp
'
' Voice 3 ADSR
'
    wTempAmp = wAmplitudeV3
    If SID_bVoice3_ControlReg.cBitGate = true Then  ' If Gate set then attack,decay,sustain
        If tVoice3_AttackCycle = true Then          ' If attack cycle
            wTempAmp = wTempAmp + wAttackV3
            If wTempAmp > cMaxLevel Then
                wTempAmp = cMaxLevel
                tVoice3_AttackCycle = false         ' If level reached, then switch to decay
            EndIf
        Else                                        ' Decay cycle
            If wTempAmp > wSustainV3 Then
                wTempAmp = wTempAmp - wDecayV3
                If wTempAmp < wSustainV3 Then
                    wTempAmp = wSustainV3
                EndIf
            EndIf
        EndIf
    Else                                            ' If Gate Flag is not set then release
        tVoice3_AttackCycle = true                  ' At next attack/decay cycle start with attack
        If wTempAmp.15 = 0 Then                     ' If wTempAmp > 0
            wTempAmp = wTempAmp - wReleaseV3
            If wTempAmp.15 = 1 Then                 ' If wTempAmp < 0
                wTempAmp = 0
            EndIf
        EndIf
    EndIf
    bEnvelopeV3 = wTempAmp.Byte1
    wAmplitudeV3 = wTempAmp
    Return
'------------------------------------------------------------------
' Data tables for attack/release
AttackRateTable:
    CData As Word 2,4,16,24,38,58,68,80,100,250,500,800,1000,3000,5000,8000
DecayReleaseRateTable:
    CData As Word 6,24,48,72,114,168,204,240,300,750,1500,2400,3000,9000,15000,24000
'------------------------------------------------------------------
' Initialise variables, PWM, and Timer interrupt
'
SID_EmuMain:
    Declare All_Digital = true
    Clear                   ' Clear all RAM before we start
    wNoise = $AA
'
' Initialise Attack, Decay, and Release envelope timings
'
    SID_bTemp = 0
    Repeat
        wAttackRate[SID_bTemp] = LRead16 AttackRateTable[SID_bTemp]
        wDecayReleaseRate[SID_bTemp] = LRead16 DecayReleaseRateTable[SID_bTemp]
        Inc SID_bTemp
    Until SID_bTemp >= 16

    tVoice1_AttackCycle = 1
    tVoice2_AttackCycle = 1
    tVoice3_AttackCycle = 1
'
' Initialise SID registers
'
    SID_bVoice1_SustainRelease = $F0
    SID_bVoice2_SustainRelease = $F0
    SID_bVoice3_SustainRelease = $F0

    SetEnvelope(0)          ' Voice 1
    SetEnvelope(1)          ' Voice 2
    SetEnvelope(2)          ' Voice 3
'
' Setup for: 10-bit PWM resolution at 62.5KHz (0 to 1023 duty cycle)
'
    T2CON = %00000100       ' Turn on Timer2 with a Prescaler value of 1:1
    PR2 = 255               ' Set PWM wFrequency
    CCPR1L = 0              ' Reset the CCPR1L register
    CCP1CON = %00001100     ' Turn on PWM Module 1 by setting bits 2 and 3 of CCP1CON
    Output PORTC.2          ' Enable the PWM output pin
'
' Configure Timer0 for:
'                       Interrupt on Timer0
'                       8-bit mode
'                       Internal clock source
'                       1:1 Prescaler
'
    T0CON = %01011111
    Clear TMR0H
    Clear TMR0L
    INTCON.2 = 0
    INTCON.5 = 1
    T0CON.7 = 1
    INTCON.7 = 1            ' Enable global interrupts




